/*
    Trance Vibe Tester (c)2006 T. R. Gipson.
    Program home page: http://tim.cexx.org/projects/vibe/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "usb.h"
#include "Unit1.h"
#include <stdio.h>
//#include <dos.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
TForm1 *AboutForm;
//---------------------------------------------------------------------------

/* Globals*/
struct usb_dev_handle* cam;     // Device handle to the device. I'm too lazy to change it from 'camera' to something else across the source...



__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
        StatusBar->SimpleText="Detecting USB Devices";
        OpenCam();

}
//---------------------------------------------------------------------------

/* Note about USB transfers: All control transfers are either in (device -> host) or out (host -> device) */

/* This data is passed in a bitfield which is the 2nd parameter of usb_control_msg.: */
/* 0xc1 = In  (type=vendor and recipient=interface) */
/* 0x41 = Out (type=vendor and recipient=interface) */


void TForm1::set_vibe(unsigned int rate)
{
        rate = (rate + rate * 256); //  0x42 -> 0x4242
        usb_control_msg(cam, 0x41, 0x00, rate, 0x0300 + (rate & 0x0F), NULL, 0, 1000);   // 0x030X where X is the low nibble of rate
        // Really should send this multiple times for flaky vibes...
        // Authentic Drmn vibes are exempt from this comment, of course :-)
}

int TForm1::OpenCam()
{
    	struct usb_bus *busses;
        //toc_entry_type* toc;
        int error=0;

        usb_set_debug(3); /* Enable libusb debug messages - recommended since both of these are still fairly 'green', not-widely-tested software */
                        /* Use something like sysinternals.com's DebugView to view the debug messages */
    	usb_init();
    	usb_find_busses();
    	usb_find_devices();
    	busses = usb_get_busses();

    	struct usb_bus *bus;
        struct usb_device *dev;

    	for (bus = busses; bus; bus = bus->next)
        {

    		for (dev = bus->devices; dev; dev = dev->next)
                {
                        StatusBar->SimpleText=dev->descriptor.idVendor;

    			if (dev->descriptor.idVendor == 0x0b49 && dev->descriptor.idProduct == 0x064f)
                        {
                                char message[128];
                                sprintf(message, "Found device with USB id %04X/%04X",dev->descriptor.idVendor, dev->descriptor.idProduct);
                                StatusBar->SimpleText=message;
                                cam=usb_open(dev);
                                if (!cam)
                                {
                                        //printf("Error opening device.\n");
                                        StatusBar->SimpleText="Error opening device.";
                                }


                                /* Now what? Claim an interface(s), I guess */


                                /* Set default configuration */
                                error=usb_set_configuration(cam, 1);  // 1? This sounds wrong, but oh well...
                                //printf("Set config: %d\n", error);

                                error=usb_claim_interface(cam, 0);

                                //printf("Found bulk endpoint %d on Configuration %d Interface %d Altsetting %d\n", bulk_endpoint, configuration, interface, altsetting);

                                /* Set alt. setting to 1...or is it 0 in libusb? */
                                error=usb_set_altinterface(cam, 0);
                                //printf("Set alt. interface: %d\n", error);

                                set_vibe(0);       // if the vibe is stuck 'on' from a previous run, turn it off

                                TrackBar1->Enabled=true;      // now let the user play with it
                                ReQuery->Enabled=false;
                                Device1->Enabled=true;
                                return true;

    			}
    		}
    	}


        /* Did NOT detect the device - ensure all related function buttons are disabled */

        TrackBar1->Enabled=false;
        ReQuery->Enabled=true;
        StatusBar->SimpleText="Did not detect a device";
        return false;
}


void __fastcall TForm1::ReQueryClick(TObject *Sender)
{
        OpenCam();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TForm1::Exit1Click(TObject *Sender)
{

        usb_release_interface(cam, 0);
        usb_close(cam);


        Form1->Close();
        exit(0);
}
//---------------------------------------------------------------------------


void __fastcall TForm1::About1Click(TObject *Sender)
{
        AboutForm->Show();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::libusbversion1Click(TObject *Sender)
{
        //struct usb_version *usb_get_version(void);
        usb_version *ver;
        ver=usb_get_version();
        char msg[128];
        sprintf(msg, "DLL version: %d.%d.%d.%d\nDriver version: %d.%d.%d.%d", ver->dll.major, ver->dll.minor, ver->dll.micro, ver->dll.nano, ver->driver.major, ver->driver.minor, ver->driver.micro, ver->driver.nano); //
        Application->MessageBox( msg ,"libusb-win32 version", MB_OK);
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------


void __fastcall TForm1::TrackBar1Change(TObject *Sender)
{
        LblVibeRate->Caption = TrackBar1->Position;
        set_vibe(TrackBar1->Position);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Device1Click(TObject *Sender)
{
        char vendor[256];
        char device[256];
        char message[512];
        usb_get_string_simple(cam, 1, vendor, 256);
        usb_get_string_simple(cam, 2, device, 256);
        sprintf(message, "Vendor string: %s\n Product string: %s\n", vendor, device);
        Application->MessageBox( message ,"Device description", MB_OK);
}
//---------------------------------------------------------------------------

